@@ -14,7 +14,10 @@
  * You should have received a copy of the GNU General Public License
  * along with TDC.  If not, see <https://www.gnu.org/licenses/>.
  *
- * Copyright: (C) Michael Lustfield <michael@lustfield.net>
+ * Copyright: Michael Lustfield <michael@lustfield.net>
+ *            Mikael Magnusson <mikachu@comhem.se>
+ *            Dave Foster <daf@minuslab.net>
+ *            Tom Cornall <t.cornall@gmail.com>
  */
 
 #include <X11/Xlib.h>
@@ -89,17 +92,18 @@ void get_params(Display *display, int argc, char *argv[]) {
   char **nargv = argv;
   char *type;
   XrmValue xrmval;
-  char *xdefaults;
+  char *home = getenv("HOME");
+  size_t xdef_len = strlen(home) + strlen("/.Xdefaults") + 1;
+  char *xdefaults = malloc(xdef_len);
 
   XrmInitialize();
   database = XrmGetDatabase(display);
   
   /* merge in Xdefaults */
-  xdefaults = malloc(strlen(getenv("HOME")) + strlen("/.Xdefaults") + 1);
   if (!xdefaults) {
     die("Memory allocation failed");
   }
-  snprintf(xdefaults, sizeof(xdefaults), "%s/.Xdefaults", getenv("HOME"));
+  snprintf(xdefaults, xdef_len, "%s/.Xdefaults", home);
   XrmCombineFileDatabase(xdefaults, &database, True);
   free(xdefaults);
 
@@ -138,7 +142,7 @@ void get_params(Display *display, int argc, char *argv[]) {
 
   /* check for version request */
   if (XrmGetResource(database, "tdc.version", "tdc.version", &type, &xrmval) == True) {
-    printf("tdc 1.6\n");
+    printf("tdc 1.8\n");
     exit(0);
   }
 
@@ -295,16 +299,16 @@ void get_calendar(char line[8][30], int month, int year) {
   char command[30];
   char buff[8][30];
 
-  snprintf(command, sizeof(command), "cal -h %d %d", month, year);
+  snprintf(command, sizeof(command), "ncal -b -h %d %d", month, year);
 
   fp = popen(command, "r");
-  while (fgets(buff[i], sizeof buff[i], fp)) {
+  while (fgets(buff[i], sizeof(buff[i]), fp)) {
     i++;
   }
   pclose(fp);
 
   for (i = 0; i < 8; i++) {
-    snprintf(line[i], sizeof(line[i]), "%-30s", buff[i]);
+    snprintf(line[i], sizeof(line[i]), "%-29.29s", buff[i]);
   }
 }
 
@@ -321,8 +325,8 @@ void findtoday(int *todayi, int *todayj, int cal_m, int cal_y) {
     pos_h = (char*) &curr_cal[2];
     pos_n = (char*) strstr(curr_cal[2], needle);
 
-    *todayi = (pos_n - pos_h) / sizeof curr_cal[2] + 2;
-    *todayj = (pos_n - pos_h) % sizeof curr_cal[2];
+    *todayi = (pos_n - pos_h) / sizeof(curr_cal[2]) + 2;
+    *todayj = (pos_n - pos_h) % sizeof(curr_cal[2]);
 
     if (getday() < 10) {
       -- * todayj;
@@ -345,6 +349,19 @@ void paintCalendar(Display *display, int cal_m, int cal_y, XftDraw *caldraw,
   glyph_w = extents.xOff;
   glyph_h = extents.height * 1.8;
 
+  if (system(NULL) == 0) {
+    char errmsg[] = "No shell available";
+    XftDrawStringUtf8(caldraw, xftcolor, font, glyph_w, glyph_h, (FcChar8*) errmsg, strlen(errmsg));
+    XFlush(display);
+    return;
+  }
+  if (system("which ncal > /dev/null 2>&1")) {
+    char errmsg[] = "ncal: not found";
+    XftDrawStringUtf8(caldraw, xftcolor, font, glyph_w, glyph_h, (FcChar8*) errmsg, strlen(errmsg));
+    XFlush(display);
+    return;
+  }
+
   get_calendar(curr_cal, cal_m, cal_y);
   findtoday(&todayi, &todayj, cal_m, cal_y);
 
